package com.example.aped;

import android.Manifest;
import android.app.AlertDialog;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.Menu;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;
import com.example.aped.communication.Communicator;
import com.example.aped.utils.ConfigurationHandler;
import com.example.aped.utils.ExternalStorageHandler;
import com.example.aped.utils.IFAVORITES;
import com.example.aped.utils.IXML;
import com.example.aped.utils.XMLHandler;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.navigation.NavigationView;
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.core.content.FileProvider;
import androidx.navigation.NavController;
import androidx.navigation.NavGraph;
import androidx.navigation.NavInflater;
import androidx.navigation.Navigation;
import androidx.navigation.ui.AppBarConfiguration;
import androidx.navigation.ui.NavigationUI;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import static android.content.pm.PackageManager.PERMISSION_GRANTED;

public class MainActivity extends AppCompatActivity {
    /** permission code for the storage permisson.*/
     private static final int STORAGE_PERMISSION_CODE = 42;

    /** What should be displayed in navigation.*/
    private AppBarConfiguration mAppBarConfiguration;

    /** To use xml instead of direct inclusion.*/
    private  IXML xml;

    /** To use delivery instead of direct inclusion.*/
    private Communicator delivery;

    /** Handler for the custom user configurations. **/
    private ConfigurationHandler configurationHandler;

    /** To use favorite instead of direct inclusion.*/
    private  IFAVORITES favorite;
    public String start = "@+id/nav_all_IOs";
    /**
     * On create method of the main activity.
     * Sets various displays and start settings.
     * @param savedInstanceState
     */
    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //checks that the permission to read and write the xml is granted
        ensurePermissions();
        setupConfigs();
        favorite = configurationHandler;
        delivery = new Communicator(configurationHandler.getAddress(),
                configurationHandler.getPort(), this);
        delivery.downloadXML();
        setContentView(R.layout.activity_main);
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        DrawerLayout drawer = findViewById(R.id.drawer_layout);
        NavigationView navigationView = findViewById(R.id.nav_view);
        // Passing each menu ID as a set of Ids because each
        // menu should be considered as top level destinations.
        mAppBarConfiguration = new AppBarConfiguration.Builder(
                R.id.nav_all_IOs, R.id.nav_your_IOs).setDrawerLayout(drawer)
                .build();
        NavController navController = Navigation.findNavController(this,
                                                        R.id.nav_host_fragment);
        NavInflater navInflater = navController.getNavInflater();
        NavGraph graph = navInflater.inflate(R.navigation.mobile_navigation);
        graph.setStartDestination(R.id.nav_your_IOs);
        navController.setGraph(graph);
        NavigationUI.setupActionBarWithNavController(this,
                                        navController, mAppBarConfiguration);
        NavigationUI.setupWithNavController(navigationView, navController);
    }

    /**
     *Adds elements to the action bar if it exists.
     * @param menu
     * @return
     */
    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    /**
     * Set up navigation.
     * @return
     */
    @Override
    public boolean onSupportNavigateUp() {
        NavController navController = Navigation.findNavController(this,
                                                        R.id.nav_host_fragment);
        return NavigationUI.navigateUp(navController, mAppBarConfiguration)
                || super.onSupportNavigateUp();
    }
    /**
     * What should happen when clicking on the respective itembutton.
     * @param item
     * @return
     */
    @Override
    public boolean onOptionsItemSelected(final @NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_set_IP:
                MaterialAlertDialogBuilder alert = new
                        MaterialAlertDialogBuilder(MainActivity.this);
                View view2 = LayoutInflater.from(MainActivity.this).inflate(R.
                        layout.alert_dialog_ip_pot, null);
                EditText ipAdress = view2.findViewById(R.id.editTextIp);
                ipAdress.setText(configurationHandler.getAddress());
                EditText port = view2.findViewById(R.id.editTextPort);
                port.setText(String.valueOf(configurationHandler.getPort()));
                alert.setView(view2);
                alert.setPositiveButton("set",
                        (dialog, which) -> {
                            configurationHandler.setAddress(ipAdress.getText()
                                    .toString());
                            configurationHandler.setPort(Integer.parseInt(port
                                    .getText().toString()));
                            Toast.makeText(getApplicationContext(),
                                    configurationHandler.getAddress()
                                            + ":" + configurationHandler.
                                            getPort() + " is set",
                                    Toast.LENGTH_SHORT).show();
                            delivery = new Communicator(
                                    configurationHandler.getAddress(),
                                    configurationHandler.getPort(),
                                    MainActivity.this);
                        });
                alert.setNegativeButton("exit",
                        (dialog, which) -> {
                            Toast.makeText(getApplicationContext(),
                                    "exit clicked",
                                    Toast.LENGTH_SHORT).show();
                            dialog.cancel();
                        });
                alert.show();
                break;

            case R.id.action_adjust_xml:
                Toast.makeText(this, "Adjust .xml",
                        Toast.LENGTH_SHORT).show();
                String externalPrivateStorageDir = null;
                try {
                    externalPrivateStorageDir = ExternalStorageHandler.
                            getExternalPrivateStorageDir(this);
                } catch (FileNotFoundException e) {
                    Log.e("MainActivity", "error while opening the "
                            + "config.xml for adjustment: " + e.getMessage());
                }
                File xmlFile = new File(externalPrivateStorageDir,
                        "config.xml");
                Uri uri = FileProvider.getUriForFile(this,
                        this.getApplicationContext().getPackageName()
                                + ".provider", xmlFile);

                Intent intent = new Intent(Intent.ACTION_VIEW);
                intent.setDataAndType(uri, "text/xml");
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
                startActivity(intent);
                break;

            case R.id.action_download_xml:
                delivery.downloadXML();
                    Toast.makeText(this, "Downloaded .xml",
                        Toast.LENGTH_SHORT).show();
                break;
            case R.id.action_upload_xml:
                try {
                    delivery.uploadXML();
                    Toast.makeText(this, "Upload .xml executed",
                            Toast.LENGTH_SHORT).show();
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                    Toast.makeText(this, "Upload .xml failed",
                            Toast.LENGTH_SHORT).show();
                }

                break;
            case R.id.action_update:
                Intent intentneustart = getIntent();
                finish();
                startActivity(intentneustart);
                break;

            default:
                Log.e("MainActivity", "unknown item: "
                        + item.toString());
                return super.onOptionsItemSelected(item);
        }
        return true;
    }

    /**
     * ensures that all the permissions needed by the application are granted.
     */
    private void ensurePermissions() {
        if (ContextCompat.checkSelfPermission(
                        this,
                        Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PERMISSION_GRANTED) {
            if (ActivityCompat.shouldShowRequestPermissionRationale(
                    this,
                    Manifest.permission.READ_EXTERNAL_STORAGE)) {

                new AlertDialog.Builder(this)
                    .setTitle("Permission needed")
                    .setMessage("This permission is needed to access the "
                        + "XML configuration file")
                    .setPositiveButton("ok", (dialog, which) ->
                        ActivityCompat.requestPermissions(
                            MainActivity.this,
                            new String[]{
                                Manifest.permission.WRITE_EXTERNAL_STORAGE},
                                    STORAGE_PERMISSION_CODE))
                    .setNegativeButton("cancel", (dialog, which) ->
                        System.exit(1))
                    .create()
                    .show();

            } else {
                ActivityCompat.requestPermissions(
                    this,
                    new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                    STORAGE_PERMISSION_CODE);
            }
        }
    }

    /**
     * handles request permission results.
     * @param requestCode the code of the requested
     * @param permissions the permissions requested
     * @param grantResults the result of the permission request
     */
    @Override
    public void onRequestPermissionsResult(
            final int requestCode,
            @NonNull final String[] permissions,
            @NonNull final int[] grantResults) {
        if (requestCode == STORAGE_PERMISSION_CODE) {
            if (grantResults.length > 0
                    && grantResults[0] == PERMISSION_GRANTED) {
                Toast.makeText(
                    this,
                    "Permission GRANTED",
                    Toast.LENGTH_LONG)
                    .show();
            } else {
                System.exit(1);
            }
        }
    }

    private void setupConfigs() {
        try {
            String externalConfigPath = ExternalStorageHandler.
                    getExternalPrivateStorageDir(MainActivity.this);
        File xmlFile = new File(externalConfigPath, "config.xml");
        File configFile = new File(externalConfigPath, "config.json");
        if (!xmlFile.exists()) {
            setDefaultXML(xmlFile);
        }
        if (!configFile.exists()) {
            setDefaultConfig(configFile);
        }
        xml = new XMLHandler(xmlFile);
        configurationHandler = new ConfigurationHandler(configFile.getPath());

        } catch (FileNotFoundException e) {
            Log.e("Main Activity", "failed to find the external "
            + "private storage during CONFIG SETUP: " + e.getMessage());
        }
    }

    /**
     *                                                                                              paul
     * @param xmlFile
     */
    private void setDefaultXML(final File xmlFile) {
        try {
            InputStream inputStream = getAssets().open(
                    "XML/default.xml");
            byte[] buffer = new byte[inputStream.available()];
            inputStream.read(buffer);
            inputStream.close();
            FileOutputStream fileOutputStream = new FileOutputStream(xmlFile);
            fileOutputStream.write(buffer);
            fileOutputStream.close();
        } catch (Exception e) {
           Log.e("MainActivity", "error while transferring the "
            + "default.xml into the config.xml: " + e.getMessage());
        }
    }

    /**
     *                                                                                          paul
     * @param xmlFile
     */
    private void setDefaultConfig(final File xmlFile) {
        try {
            InputStream inputStream = getAssets().open(
                    "defaultConfig.json");
            byte[] buffer = new byte[inputStream.available()];
            inputStream.read(buffer);
            inputStream.close();
            FileOutputStream fileOutputStream = new FileOutputStream(xmlFile);
            fileOutputStream.write(buffer);
            fileOutputStream.close();
        } catch (Exception e) {
            Log.e("MainActivity", "error while transferring the "
                    + "defaultConfig.json into the config.json: "
                    + e.getMessage());
        }
    }

    /**
     * Getter method for favorite.
     * @return favorite get.
     */
    public IFAVORITES getFavorite() {
        return favorite;
    }

    /**
     * Setter method for favorite.
     * @param pFavorite set.
     */
    public void setFavorite(final IFAVORITES pFavorite) {
        this.favorite = pFavorite;
    }

    /**
     * Getter method for xml.
     * @return xml get.
     */
    public IXML getXml() {
        return xml;
    }

    /**
     * Setter method for xml.
     * @param pXml set.
     */
    public void setXml(final IXML pXml) {
        this.xml = pXml;
    }

    /**
     * Getter method for delivery.
     * @return delivery get.
     */
    public Communicator getDelivery() {
        return delivery;
    }

    /**
     * Setter method for delivery.
     * @param pDelivery set.
     */
    public void setDelivery(final Communicator pDelivery) {
        this.delivery = pDelivery;
    }

    /**
     * Getter method for configurationHandler.
     * @return configurationHandler get.
     */
    public ConfigurationHandler getConfigurationHandler() {
        return configurationHandler;
    }

    /**
     * Setter method for configurationHandler.
     * @param pConfigurationHandler set.
     */
    public void setConfigurationHandler(final ConfigurationHandler
                                                pConfigurationHandler) {
        this.configurationHandler = pConfigurationHandler;
    }
}
