package com.example.aped.communication;

import com.android.volley.NetworkResponse;
import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.toolbox.HttpHeaderParser;

import java.util.Map;

/**
 * Custom {@link Request} to enable recieving files from the raspberry pi.
 * Credit: https://mobikul.com/downloading-files-using-volley/
 */
public class InputStreamVolleyRequest extends Request<byte[]> {

    /** Listerner for response callback. **/
    private final Response.Listener<byte[]> listener;

    /** Headers of the response recieved from the raspberry.**/
    private Map<String, String> responseHeaders;

    /**
     * Constructor.
     * @param method {@link Request.Method}
     * @param url the API url to send the request to
     * @param pListener the listener for the response callback
     * @param errorListener the listener for an error callback
     */
    public InputStreamVolleyRequest(final int method,
                                final String url,
                                final Response.Listener<byte[]> pListener,
                                final Response.ErrorListener errorListener) {
        super(method, url, errorListener);
        setShouldCache(false);
        this.listener = pListener;
    }

    /**
     * Parses the response into a byte array.
     * @param response the response recieved from the raspberry
     * @return the parsed response
     */
    @Override
    protected Response<byte[]> parseNetworkResponse(
            final NetworkResponse response) {
        responseHeaders = response.headers;
        return Response.success(
                response.data, HttpHeaderParser.parseCacheHeaders(response)
        );
    }

    /**
     * Delivers the recieved response to the listener.
     * @param response the parsed response
     */
    @Override
    protected void deliverResponse(final byte[] response) {
        listener.onResponse(response);
    }
}
