package com.example.aped.ui.visualization;


import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import com.example.aped.MainActivity;
import com.example.aped.R;
import java.util.ArrayList;
import java.util.List;


public class MainListViewAdapter extends BaseAdapter {
    /** Include the MainActivity.*/
    private MainActivity mainActivity;
    /** Which view was selected, 0 for all IOs, 1 for favorite IOs.*/
    private int favoriteMode;
    /** Name of the device for the current run.*/
    private String deviceName;
    /** Image for favorite or not.*/
    private int favoriteImage;
    /** List of favorites existing when opening the favorites view.*/
    private List<String> favoritenList;

    /**
     * Transfer of data to MainListViewAdapter.
     * @param pMainActivity
     * @param pFavoriteMode
     */
    public MainListViewAdapter(final MainActivity pMainActivity,
                               final int pFavoriteMode) {
        this.mainActivity = pMainActivity;
        this.favoriteMode = pFavoriteMode;
    }

    /**
     * Set number of list elements.
     * @return devices number
     */
    @Override
    public int getCount() {
        int devices;
        if (favoriteMode == 0) {
            devices = mainActivity.getXml().getDeviceNames().size();
        } else {
            devices = mainActivity.getFavorite().getFavorites().size();
            favoritenList = new ArrayList<>(mainActivity.getFavorite()
                    .getFavorites());
        }
        return devices;
    }

    /**
     * This function is used to retrieve the data item associated.
     *  With the specified position in the data set to get the corresponding
     *  data of the specified location in the data collection.
     * @param position
     * @return
     */
    @Override
    public Object getItem(final int position) {

        return null;
    }

    /**
     * As for the getItemId it returns the corresponding item element ID.
     * The function returns a long item position value to the adapter.
     * @param position
     * @return
     */
    @Override
    public long getItemId(final int position) {

        return 0;
    }

    /**
     * Device name display and favorite button and data sharing for state
     * visitation.
     * @param i
     * @param pView
     * @param viewGroup
     * @return view
     */
    @Override
    public View getView(final int i, final View pView,
                        final ViewGroup viewGroup) {
        View view = mainActivity.getLayoutInflater().inflate(R.layout
                .listview_layout, null);
        TextView textViewName = view.findViewById(R.id.textName);
        LinearLayout linearLayoutPin = view.findViewById(R.id
                .idLinearLayoutPins);
        ImageView buttonViewFavoriten = view.findViewById(R.id
                .imageButtonFavoriten);
        if (favoriteMode == 0) {
            deviceName = mainActivity.getXml().getDeviceNames().get(i);
        } else {
            deviceName = mainActivity.getFavorite().getFavorites().get(i);
        }

        if (mainActivity.getFavorite().getFavorites().contains(deviceName)) {
            favoriteImage = android.R.drawable.btn_star_big_on;
        } else {
            favoriteImage = android.R.drawable.btn_star_big_off;
        }
        buttonViewFavoriten.setOnClickListener(view1 -> {
            if (favoriteMode == 0) {
                deviceName = mainActivity.getXml().getDeviceNames().get(i);
            } else {
                deviceName = favoritenList.get(i);
            }
            if (mainActivity.getFavorite().getFavorites()
                    .contains(deviceName)) { mainActivity.getFavorite()
                    .removeFavorite(deviceName);
                favoriteImage = android.R.drawable.btn_star_big_off;
                buttonViewFavoriten.setImageResource(favoriteImage);
            } else {
                mainActivity.getFavorite().addFavorite(deviceName);
                favoriteImage = android.R.drawable.btn_star_big_on;
                buttonViewFavoriten.setImageResource(favoriteImage);
            }
        });
        PinView pinView = new PinView(mainActivity, deviceName);
        for (int j = 0; j < ((List<String>) mainActivity.getXml()
                .getPort(deviceName).get("pins")).size(); j++) {
            linearLayoutPin.addView(pinView.getView(j));
        }
        String name = null;
        switch ((mainActivity.getXml().getPort(deviceName)).get("protocol").
                toString()) {
            case "PWM":
                name = ((mainActivity.getXml().getPort(deviceName))
                        .get("protocol").toString() + ": " + deviceName
                        + " (" + (mainActivity.getXml().getPort(deviceName))
                        .get("frequency").toString() + "Hz)");
                break;
            default:
                name = ((mainActivity.getXml().getPort(deviceName)).
                        get("protocol").toString() + ": " + deviceName);
                break;
        }
        textViewName.setText(name);
        buttonViewFavoriten.setImageResource(favoriteImage);
        return view;
    }
}
