package com.example.aped.ui.visualization;

import android.text.Html;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;
import com.example.aped.MainActivity;
import com.example.aped.R;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.slider.RangeSlider;
import org.json.JSONException;
import org.json.JSONObject;
import java.util.ArrayList;
import java.util.List;

public class PinView {
    /** Include the MainActivity.*/
    private MainActivity mainActivity;
    /** Image for state.*/
    private int image;
    /** Values of the pins.*/
    private List<String> pinValues = new ArrayList<>();
    /** Device name.*/
    private  String deviceName;

    /**
     * Transfer of data to PinView.
     * @param pMainActivity
     * @param pDeviceName
     */
    public PinView(final MainActivity pMainActivity, final String pDeviceName) {
        this.mainActivity = pMainActivity;
        this.deviceName = pDeviceName;
    }

    /**
     * Processing of the data in displayed texts of the pins frequencies and
     * dutycycle as well as the status lamps.
     * @param i
     * @return view
     */
    public View getView(final int i) {
        View view = mainActivity.getLayoutInflater().inflate(R.layout
                .pin_listview_layout, null);
        TextView textViewPin = view.findViewById(R.id.textPin);
        textViewPin.setText(((List<String>) (mainActivity.getXml().
                getPort(deviceName)).get("pins")).get(i));
        mainActivity.getDelivery().read(deviceName, response -> {
            displayLamp(response, view, i);
            if ((mainActivity.getXml().getPort(deviceName)).get("protocol")
                    .toString().equals("PWM")) {
                textViewPin.setText(Html.fromHtml(((List<String>)
                        (mainActivity.getXml().getPort(deviceName)).get("pins"))
                        .get(i) + " <b>  (DC: " + pinValues.get(i) + "%) </b>"
                ));
            }
        });

        view.setOnClickListener(view1 -> {
            switch ((mainActivity.getXml().getPort(deviceName)).get("protocol")
                    .toString()) {
                case "DO":
                    try {
                        String setState;
                        if (pinValues.get(i).equals("0")) {
                            setState = "\"1\"";
                        } else {
                            setState = "\"0\"";
                        }
                        JSONObject jsonObject = new JSONObject(
                             "{\"output\":{\"" + ((List<String>) (mainActivity
                                     .getXml().getPort(deviceName)).get("pins"))
                                     .get(i) + "\":" + setState + "}}");
                        mainActivity.getDelivery().write(deviceName,
                                jsonObject, response -> displayLamp(response,
                                        view1, i));
                    } catch (JSONException e) {
                        Log.e("PinView",
                             "Error while generating JSON object: "
                              + e.getMessage());
                    }
                    break;
                case "PWM":
                    MaterialAlertDialogBuilder alert =
                            new MaterialAlertDialogBuilder(mainActivity);
                    View viewAlert = mainActivity.getLayoutInflater()
                            .inflate(R.layout.alert_dialog_duty, null);
                    RangeSlider readDuty = viewAlert.findViewById(R.id
                            .range_slider_duty);
                    readDuty.setValues(Float.valueOf(pinValues.get(i)
                            .equals("None") ? "0" : pinValues.get(i))
                    );
                    alert.setView(viewAlert);
                    alert.setPositiveButton("set", (dialog, which) -> {
                        int value = readDuty.getValues().get(0).intValue();
                        try {
                            JSONObject jsonObject = new JSONObject(
                                 "{\"output\":{\"" + ((List<String>)
                                 (mainActivity.getXml().getPort(deviceName))
                                 .get("pins")).get(i) + "\":\"" + value
                                         + "\"}}");
                            mainActivity.getDelivery().write(deviceName,
                                 jsonObject, response -> {
                                 displayLamp(response, view1, i);
                                 textViewPin.setText(Html.fromHtml((
                                      (List<String>) (mainActivity.getXml()
                                      .getPort(deviceName)).get("pins")).get(i)
                                      + " <b>  (DC: " + pinValues.get(i)
                                      + "%) </b>"));
                                 });
                            } catch (JSONException e) {
                                Log.e("PinView",
                                    "Error while generating JSON object: "
                                    + e.getMessage());
                            }
                            dialog.cancel();
                        });
                    alert.setNegativeButton("exit", (dialog,
                                                     which) -> dialog.cancel());
                    alert.show();
                    break;
                default:
                    Toast.makeText(mainActivity, deviceName
                            + " is not an output ", Toast.LENGTH_SHORT).
                            show();
                    break;
            }


        });
        return view;
    }

    /**
     * Method for the selection of the lamps.
     * @param response
     * @param view
     * @param i
     */
    private void displayLamp(final JSONObject response, final View view,
                             final int i) {
        ImageView imageView = view.findViewById(R.id.images);
        try {
            JSONObject state = response.getJSONObject("state");
            if (pinValues.size() != 0) {
                pinValues.clear();
            }
            for (String pinName: ((List<String>) mainActivity.getXml()
                    .getPort(deviceName).get("pins"))) {
                pinValues.add(state.getString(pinName));
            }
            switch (pinValues.get(i)) {
                case "1":
                    image = R.drawable.green_signal;
                    break;
                case "0":
                    image = R.drawable.red_signal;
                    break;
                case "100":
                    image = R.drawable.green_signal;
                    break;
                default:
                    image = R.drawable.off_signal;
                    break;
            }
            imageView.setImageResource(image);
        } catch (JSONException e) {
            Log.e("PinView", "Error while generating JSON object: "
                 + e.getMessage());
        }
    }
}
