package com.example.aped.utils;

import android.util.Log;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

/**
 * Class to handle operations on the config.xml files.
 */
public class XMLHandler implements IXML {

    /** the root of the XMl file. **/
    private Document root;

    /**
     * constructor for the XMLHandler.
     * @param xmlFile the XML file
     * @throws ParserConfigurationException the XML parser configuration failed
     * @throws IOException the XML file could not be accessed
     * @throws SAXException the XML parse failed
     */
    public XMLHandler(final File xmlFile) {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            root = builder.parse(xmlFile);
        } catch (ParserConfigurationException e) {
            Log.e("XMLHandler", "error while parsing the XML file: "
                    + e.getMessage());
        } catch (IOException e) {
            Log.e("XMLHandler", "error while reading from the XML file: "
                    + e.getMessage());
        } catch (SAXException e) {
            Log.e("XMLHandler", "general SAX error: " + e.getMessage());
        }
    }

    /**
     * reads the device names from the XML file.
     * @return the device names as a list of strings
     */
    @Override
    public List<String> getDeviceNames() {
        List<String> returnList = new ArrayList<>();
        NodeList devices = root.getElementsByTagName("Device");
        for (int i = 0; i < devices.getLength(); i++) {
           returnList.add(((Element) devices.item(i)).getAttribute("name"));
        }
        return returnList;
    }

    /**
     * reads the port information from the XML file.
     * @param deviceName the name of the relevant device
     * @return the port information as a dictionary {'protocol','pins'}
     */
    @Override
    public Dictionary<String, Object> getPort(final String deviceName) {
        Dictionary<String, Object> returnDictionary = new Hashtable<>();
        XPathFactory xPathFactory = XPathFactory.newInstance();
        XPath xPath = xPathFactory.newXPath();
        try {
            XPathExpression xPathExpression = xPath.compile(
                    "//Device[@name='" + deviceName + "']/Port");
            Element result = (Element) xPathExpression.evaluate(
                    root, XPathConstants.NODE);
            returnDictionary.put("protocol", result.getAttribute("protocol"));
            returnDictionary.put("frequency", result.getAttribute("frequency"));
            NodeList childNodes = result.getChildNodes();
            List<String> pins = new ArrayList<>();
            for (int i = 0; i < childNodes.getLength(); i++) {
                Node childNode = childNodes.item(i);
                if (childNode.getNodeName().equals("Pin")) {
                   pins.add(childNode.getTextContent());
                }
            }
            returnDictionary.put("pins", pins);
        } catch (XPathExpressionException e) {
            Log.e(
                    "XMLHandler",
                    "the XPath for getting the value info has errors:"
                            + e.getMessage()
            );
        }
        return returnDictionary;
    }
}
